/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */
package cz.xelfi.quoridor.visidor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Fence;
import cz.xelfi.quoridor.Fence.Orientation;
import cz.xelfi.quoridor.IllegalPositionException;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.Player;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
final class Viewer extends JPanel implements MouseMotionListener {
    private Board board;
    private int fieldSize;
    private int xdelta;
    private int ydelta;
    private Orientation lastOrientation;

    public Viewer() {
        this(Board.empty());
    }

    private Viewer(Board b) {
        this.board = b;
        addMouseMotionListener(this);
    }


    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws IllegalPositionException {
        Board b = Board.empty();
        for (int i = 1; i <= 8; i++) {
            b = b.apply(Move.fence('A', i, Orientation.values()[i % 2]));
        }
        for (int i = 1; i <= 8; i++) {
            b = b.apply(Move.fence('H', i, Orientation.values()[(i + 1) % 2]));
        }

        JFrame f = new JFrame();
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.add(new Viewer(b));
        f.pack();
        f.setVisible(true);
    }

    @Override
    public Dimension getPreferredSize() {
        return new Dimension(450, 450);
    }

    @Override
    protected void paintComponent(Graphics gg) {
        assert EventQueue.isDispatchThread();
        
        fieldSize = Math.min(getSize().width, getSize().height) / 9;
        int fifth = fieldSize / 10;
        Graphics2D g = (Graphics2D) gg;
        
        xdelta = (getSize().width - fieldSize * 9) / 2;
        ydelta = (getSize().height - fieldSize * 9) / 2;
        
        g.translate(xdelta, ydelta);

        g.setColor(Color.lightGray);
        for (int i = 0; i < 9; i++) {
            for (int j = 0; j < 9; j++) {
                final Rectangle r = new Rectangle(i * fieldSize, j * fieldSize, fieldSize, fieldSize);
                g.fillRect(r.x + fifth, r.y + fifth, r.width - fifth * 2, r.height - fifth * 2);
            }
        }


        g.setColor(Color.BLACK);
        for (Fence f : board.getFences()) {
            int column = (f.getColumn() - 'A') + 1;
            int row = 9 - f.getRow();
            Orientation orient = f.getOrientation();
            drawFence(orient, column, row, g, true);
        }

        int cnt = 0;
        Color[] colors = { Color.WHITE, Color.BLACK, Color.ORANGE, Color.MAGENTA };
        int diametr = fieldSize - fifth * 4;
        for (Player p : board.getPlayers()) {
            int column = p.getColumn();
            int row = 8 - p.getRow();
            final Rectangle r = new Rectangle(
                column * fieldSize + 2 * fifth,
                row * fieldSize + 2 * fifth,
                diametr,
                diametr
            );
            g.setColor(colors[cnt]);
            g.fillOval(r.x, r.y, r.width, r.height);
            if (p == board.getCurrentPlayer()) {
                g.setColor(Color.lightGray);
                g.fillOval(r.x + r.width / 3, r.y + r.height / 3, r.width / 3, r.height / 3);
            }
            cnt++;
        }

        g.setColor(Color.BLACK);
        final Font f = new Font("Monospace", Font.BOLD, fifth * 2);
        g.setFont(f);
        for (int i = 0; i < 8; i++) {
            final char ch = (char) ('A' + i);
            g.drawString(Character.toString(ch), i * fieldSize + fieldSize - fifth, fifth + f.getSize() - f.getBaselineFor(ch));
        }
        for (int i = 0; i < 8; i++) {
            String s = "" + (8 - i);
            g.drawString(s, fifth, i * fieldSize + fieldSize - fifth + f.getSize() - f.getBaselineFor(s.charAt(0)));
        }
    }

    private void drawFence(Orientation orient, int column, int row, Graphics2D g, boolean fill) throws IllegalStateException {
        int fifth = fieldSize / 10;
        int w, h;
        switch (orient) {
            case HORIZONTAL: w = fieldSize - fifth; h = fifth; break;
            case VERTICAL: w = fifth; h = fieldSize - fifth; break;
            default: throw new IllegalStateException();
        }
        Rectangle r = new Rectangle(
            column * fieldSize - w,
            row * fieldSize - h,
            2 * w,
            2 * h
        );
        if (fill) {
            g.fill(r);
        } else {
            g.draw(r);
        }
    }

    public void mouseDragged(MouseEvent e) {
    }

    public void mouseMoved(MouseEvent e) {
        int x = Math.round(((float)(e.getX() - xdelta)) / fieldSize);
        int y = Math.round(((float)(e.getY() - ydelta)) / fieldSize);
        if (x <= 0 || x >= 9) {
            return;
        }
        if (y <= 0 || y >= 9) {
            return;
        }
        
        int dx = (e.getX() - xdelta) % fieldSize;
        int dy = (e.getY() - ydelta) % fieldSize;
        
        int fifth = fieldSize / 10;
        boolean outOfX = dx > fifth && dx < fieldSize - fifth;
        boolean outOfY = dy > fifth && dy < fieldSize - fifth;
        if (outOfX && outOfY) {
            return;
        }
        if (!outOfX && !outOfY) {
            if (lastOrientation == null) {
                return;
            }
        } else {
            if (outOfX) {
                lastOrientation = Orientation.HORIZONTAL;
            } else {
                lastOrientation = Orientation.VERTICAL;
            }
        }
        
        BufferedImage img = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_ARGB);
        final Graphics2D d2 = img.createGraphics();
        d2.setBackground(getBackground());
        d2.clearRect(0, 0, getWidth(), getHeight());
        paintComponent(d2);
        
        try {
            Board newBoard = board.apply(Move.fence((char)('A' + x - 1), 9 - y, lastOrientation));
            drawFence(lastOrientation, x, y, d2, true);
        } catch (IllegalPositionException ex) {
            // can't place fence
            drawFence(lastOrientation, x, y, d2, false);
        }
        
        
        getGraphics().drawImage(img, 0, 0, null);
    }

}
